@@ -1,4 +1,4 @@
 from ._version import __version__, __version_info__
-from .experiment import *
+from .experiments import *
 from .trainer import Trainer
 from .options import set_options
