@@ -1,4 +1,5 @@
 import os
+from typing import Optional
 
 import numpy as np
 import torch
@@ -13,17 +14,17 @@ class BaseModel(nn.Module):
 
     def __init__(self):
         super().__init__()
-        self.global_step = 0
+        self._global_step = 0
 
     def increment_global_step(self):
         """Increments global step by 1."""
-        self.global_step += 1
+        self._global_step += 1
 
-    def get_device(self):
+    def get_device(self) -> torch.device:
         """Returns model device."""
         return get_module_device(self)
 
-    def checkpoint(self, ckpt_folder, max_ckpt=None):
+    def checkpoint(self, ckpt_folder: str, max_ckpt: Optional[int] = None):
         """Saves model checkpoint and optionally deletes old ones.
 
         Args:
@@ -53,7 +54,10 @@ class BaseModel(nn.Module):
             except OSError:
                 pass
 
-    def load(self, ckpt_folder, device=None, step=None):
+    def load(self,
+             ckpt_folder: str,
+             device: Optional[torch.device] = None,
+             step: Optional[int] = None):
         """Loads model from checkpoint.
 
         Args:
@@ -67,21 +71,26 @@ class BaseModel(nn.Module):
         """
         if step is None:
             filenames, numbers = checkpoints_in_folder(ckpt_folder)
-            ckpt_name = filenames[np.argmax(numbers)]   # get latest checkpoint
+            ckpt_name = filenames[np.argmax(numbers)]  # get latest checkpoint
             step = max(numbers)
         else:
             ckpt_name = "model_{}.pt".format(step)
         print("Loading model checkpoint at step {}...".format(step))
         path = os.path.join(ckpt_folder, ckpt_name)
         self.load_state_dict(torch.load(path, map_location=device))
-        self.global_step = step
+        self._global_step = step
         print("Loaded.")
 
+    @property
+    def global_step(self) -> int:
+        """Global step."""
+        return self._global_step
+
 
 class BaseGenerativeModel(BaseModel):
     """Base class for generative models."""
 
-    def sample_prior(self, n_imgs, **kwargs):
+    def sample_prior(self, n_imgs: int, **kwargs):
         """Samples an observation from the generative model.
 
         Args:
